@@ -2,6 +2,20 @@
 #include "metis.h"
 #include "Metis_utils.h"
 
+
+
+/** 
+ * Establish a fill-reducing permutation for the sparse symmetric
+ * matrix of order n represented by the column pointers Tp and row
+ * indices Ti.
+ * 
+ * @param n  order of the sparse symmetric matrix
+ * @param Tp  column pointers (total length n + 1)
+ * @param Ti  row indices (total length Tp[n])
+ * @param Perm array of length n to hold the permutation
+ * @param iPerm array of length n to hold the inverse permutation
+ * 
+ */
 void ssc_metis_order(int n, const int Tp [], const int Ti [],
 		     int Perm[], int iPerm[])
 {
@@ -12,8 +26,13 @@ void ssc_metis_order(int n, const int Tp [], const int Ti [],
 	*xadj = Calloc(n+1, idxtype),
 	*adj = Calloc(2 * (Tp[n] - n), idxtype);
 
+				/* check row indices for correct range */
+    for (j = 0; j < Tp[n]; j++)
+      if (Ti[j] < 0 || Ti[j] >= n)
+	error(_("row index Ti[%d] = %d is out of range [0,%d]"),
+	      j, Ti[j], n - 1);
 				/* temporarily use perm to store lengths */
-    memset(perm, 0, sizeof(idxtype) * n);
+    AZERO(perm, n);
     for (j = 0; j < n; j++) {
 	int ip, p2 = Tp[j+1];
 	for (ip = Tp[j]; ip < p2; ip++) {
